#include "ptrfelinit.h"
#include "pglobalt.h"
#include "xfile.h"
#include "stochdrivert.h"
#include "stacktrace.h"

void ptrfel_init (int argc,const char *argv[], stochdrivert *stochd)
{
  char name[1100];
  XFILE *in;
  
  set_prgname(argv[0]);
  //  creation of main objects
  Ptp = new pprobdesct;

  Tp = new probdesct;
  Gtt = new gtopology;
  Tt = new transtop;
  Tm = new transmat;
  Tc = new transcrsec;
  Tb = new transbclc;
  Outdt = new outdrivert;
  
  /*
  if (Myrank==0){
    char bd;
    scanf ("%c",&bd);
  }
  MPI_Barrier (MPI_COMM_WORLD);
  */

  if (process_argst(argc, argv))
  {
    delete Tp;
    delete Gtt;
    delete Tt;
    delete Tm;
    delete Tc;
    delete Tb;
    delete Outdt;
    delete stochd;
    abort();
  }

  //  names of input files
  strcpy (name,argv[1]);
  sprintf (&name[strlen (argv[1])],"%d.in",Myrank+1);
  in = xfopen (name,"r");
  if (in==NULL){
    par_print_err(Myrank+1, proc_namet, "input file cannot be opened.",__FILE__, __LINE__, __func__);
    delete Tp;
    delete Gtt;
    delete Tt;
    delete Tm;
    delete Tc;
    delete Tb;
    delete Outdt;
    delete stochd;
    abort();
  }
  in->maxlnsize = 1000;
  in->warning = 1;
  in->ignorecase = 1;
  if ((Tp->kwdsw == nokwd) || (Tp->kwdsw == kwd_outd))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;

  //  number of subdomain
  xfscanf (in,"%k%ld","domain_num",&Ndom);
  Ndom--;


  // **************************
  // **  basic problem data  **
  // **************************
  //  basic problem data
  Tp->read (in);
  
  //  creation of additional object
  Psolt = new psolver (Nproc,Myrank,proc_namet,nameLengtht);
  //  reading data about parallel solver
  Psolt->read (in,Gtt,Ndom,Tp->tstorkm,Mesprt);
  
  if (Tp->kwdsw == kwd_full)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;

  char buf[1001];
  strcpy (buf,argv[1]);
  sprintf (&buf[strlen (argv[1])],"%d.log",Myrank+1);
  Outt = fopen (buf,"w");
  if (Outt==NULL){
    par_print_err (Myrank+1,proc_namet,"log file cannot be opened",__FILE__, __LINE__, __func__);
  }
 
  //  correspondence between processors and subdomains
  Psolt->procdomcorr ();

  
  // **************************************
  // **  input of node and element data  **
  // **************************************

  //  input of node and element data
  Tt->read (in);

  //  initiation of parallel solver
  Psolt->initiate (Gtt,argc,argv);
  //  reading of the array ltg (local-to-global correspondence)
  Psolt->read_ltg (in,Gtt,Outt);
  fprintf (Outt,"\n Nproc  %d",Nproc);
  fprintf (Outt,"\n Ndom   %d",Ndom);
  fprintf (Outt,"\n Myrank %d",Myrank);

  if (Tp->tprob == discont_nonstat_problem || Tp->tprob == discont_nonlin_nonstat_problem){
    Gtt->auto_subdom_detection (Outt);
    Tt->alloc_enodes ();
    Tt->alloc_edges ();
  }

  // *************************************
  //  generation of ordering of unknowns
  //  (generation of code numbers)
  //  number of degrees of freedom
  // *************************************
  Ndoft=Psolt->ordering (Gtt,Outt,proc_namet);

  if (Mesprt==1)
    fprintf (stdout,"\n number of degrees of freedom    %ld",Ndoft);

  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    Gtt->read_gf (in);
  }

  //  input of material characteristics
  Tm->read (in);
  
  //int. points allocation and initiation
  Tm->intpointalloc ();
  Tm->intpointinit ();

  //  input of cross section characteristics
  Tc->read (in);
  
  //  input of boundary conditions
  Tb->read (in);
  
  //  prescribed values on elements investigation
  //Tt->elemprescvalue();
  //  searching for elements with nodes where source is defined
  Tb->elemsource ();
  
  
  //  global vectors allocations
  Lsrst = new lhsrhst;
  Lsrst->alloc ();  
  Lsrst->initcond (in);
  
  if (Tp->stochasticcalc){
    stochd->read (in);
  };

  if (Tp->tprob == discont_nonstat_problem || Tp->tprob == discont_nonlin_nonstat_problem){

    Tt->enodes_init ();
    Tt->edge_init ();
    //  definition of material types and id on nodes
    //Tt->node_materials ();    
  }
  //Tt->node_materials ();
  //  toto je pokusne umisteni

  if ((Tp->kwdsw == nokwd) || (Tp->kwdsw == kwd_pd))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;

  // input of output description
  Outdt->read(in);

  if (Tp->kwdsw == kwd_full)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;

  //  names of output files
  strcpy (name,Outdt->outfn);
  sprintf (&name[strlen (Outdt->outfn)],"%d.out",Myrank+1);
  strcpy (Outdt->outfn, name);
  if (Outdt->gf != grftt_no)
  {
    strcpy (name,Outdt->outgrfn);
    sprintf (&name[strlen (Outdt->outgrfn)],"%d",Myrank+1);
    strcpy (Outdt->outgrfn, name);
  }
  if (Outdt->ndiag > 0)
  {
    strcpy (name,Outdt->outdiagfn);
    sprintf (&name[strlen (Outdt->outdiagfn)],"%d.dat",Myrank+1);
    strcpy (Outdt->outdiagfn, name);
  }  

  //  nodes allocation
  Tt->alloc_nodes ();

  fflush(stdout);
  fflush(stderr);

  Ptp->shift_indices ();
  xfclose (in);

}
